@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# This is a kernel build script for Fedora 12's 2.6.32 kernel.
+# This is a kernel build script for Fedora 11's 2.6.30 kernel.
 #
 
 die () {
@@ -10,23 +10,24 @@ die () {
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 
-if [ ! -r kernel-2.6.32.10-90.fc12.src.rpm ]
+if [ ! -r kernel-2.6.30.10-105.2.23.fc11.src.rpm ]
 then
-    wget http://ftp.riken.jp/Linux/fedora/updates/12/SRPMS/kernel-2.6.32.10-90.fc12.src.rpm || die "Can't download source package."
+    wget http://ftp.riken.jp/Linux/fedora/updates/11/SRPMS/kernel-2.6.30.10-105.2.23.fc11.src.rpm || die "Can't download source package."
 fi
-rpm -ivh kernel-2.6.32.10-90.fc12.src.rpm || die "Can't install source package."
+rpm --checksig kernel-2.6.30.10-105.2.23.fc11.src.rpm || die "Can't verify signature."
+rpm -ivh kernel-2.6.30.10-105.2.23.fc11.src.rpm || die "Can't install source package."
 
 cd /root/rpmbuild/SOURCES/ || die "Can't chdir to /root/rpmbuild/SOURCES/ ."
-if [ ! -r ccs-patch-1.7.2-20100401.tar.gz ]
+if [ ! -r ccs-patch-1.7.2-20100412.tar.gz ]
 then
-    wget http://sourceforge.jp/frs/redir.php?f=/tomoyo/43375/ccs-patch-1.7.2-20100401.tar.gz || die "Can't download patch."
+    wget http://sourceforge.jp/frs/redir.php?f=/tomoyo/43375/ccs-patch-1.7.2-20100412.tar.gz || die "Can't download patch."
 fi
 
 cd /root/rpmbuild/SPECS/ || die "Can't chdir to /root/rpmbuild/SPECS/ ."
 cp -p kernel.spec ccs-kernel.spec || die "Can't copy spec file."
 patch << "EOF" || die "Can't patch spec file."
---- ccs-kernel.spec	2010-03-23 06:18:08.000000000 +0900
-+++ ccs-kernel.spec	2010-03-30 20:34:44.200183205 +0900
+--- ccs-kernel.spec	2010-02-11 15:43:46.000000000 +0900
++++ ccs-kernel.spec	2010-02-13 17:41:56.903388784 +0900
 @@ -15,7 +15,7 @@
  # that the kernel isn't the stock distribution kernel, for example,
  # by setting the define to ".local" or ".bz123456"
@@ -36,7 +37,7 @@ patch << "EOF" || die "Can't patch spec file."
  
  # fedora_build defines which build revision of this kernel version we're
  # building. Rather than incrementing forever, as with the prior versioning
-@@ -410,6 +410,11 @@
+@@ -393,6 +393,11 @@
  # to versions below the minimum
  #
  
@@ -48,16 +49,16 @@ patch << "EOF" || die "Can't patch spec file."
  #
  # First the general kernel 2.6 required versions as per
  # Documentation/Changes
-@@ -445,7 +450,7 @@
+@@ -428,7 +433,7 @@
  # Packages that need to be installed before the kernel is, because the %post
  # scripts use them.
  #
--%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, kernel-firmware >= %{rpmversion}-%{pkg_release}, grubby >= 7.0.4-1
-+%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, grubby >= 7.0.4-1
- %if %{with_dracut}
- %define initrd_prereq  dracut >= 002 xorg-x11-drv-ati-firmware
- %else
-@@ -481,7 +486,7 @@
+-%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, mkinitrd >= 6.0.61-1, kernel-firmware >= %{rpmversion}-%{pkg_release}, /sbin/new-kernel-pkg
++%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, mkinitrd >= 6.0.61-1, /sbin/new-kernel-pkg
+ 
+ #
+ # This macro does requires, provides, conflicts, obsoletes for a kernel package.
+@@ -456,7 +461,7 @@
  AutoProv: yes\
  %{nil}
  
@@ -66,7 +67,7 @@ patch << "EOF" || die "Can't patch spec file."
  Group: System Environment/Kernel
  License: GPLv2
  URL: http://www.kernel.org/
-@@ -886,7 +891,7 @@
+@@ -993,7 +998,7 @@
  Provides: kernel-devel-uname-r = %{KVERREL}%{?1:.%{1}}\
  AutoReqProv: no\
  Requires(pre): /usr/bin/find\
@@ -75,18 +76,18 @@ patch << "EOF" || die "Can't patch spec file."
  This package provides kernel headers and makefiles sufficient to build modules\
  against the %{?2:%{2} }kernel package.\
  %{nil}
-@@ -1439,6 +1444,10 @@
+@@ -1692,6 +1697,10 @@
  
- # END OF PATCH APPLICATIONS ====================================================
+ # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.7.2-20100401.tar.gz
-+patch -sp1 < patches/ccs-patch-2.6.32-fedora-12.diff
++tar -zxf %_sourcedir/ccs-patch-1.7.2-20100412.tar.gz
++patch -sp1 < patches/ccs-patch-2.6.30-fedora-11.diff
 +
  %endif
  
  # Any further pre-build tree manipulations happen here.
-@@ -1465,6 +1474,9 @@
+@@ -1718,6 +1727,9 @@
  for i in *.config
  do
    mv $i .config
@@ -103,4 +104,8 @@ echo ""
 echo "Edit /root/rpmbuild/SPECS/ccs-kernel.spec if needed, and run"
 echo "rpmbuild -bb /root/rpmbuild/SPECS/ccs-kernel.spec"
 echo "to build kernel rpm packages."
+echo ""
+echo "I'll start 'rpmbuild -bb --target i586 --with baseonly --without debug --without debuginfo /root/rpmbuild/SPECS/ccs-kernel.spec' in 30 seconds. Press Ctrl-C to stop."
+sleep 30
+exec rpmbuild -bb --target i586 --with baseonly --without debug --without debuginfo /root/rpmbuild/SPECS/ccs-kernel.spec
 exit 0
