@@ -14,12 +14,13 @@ if [ ! -r kernel-2.6.20-1.2320.fc5.src.rpm ]
 then
     wget http://ftp.riken.jp/Linux/fedora/core/updates/5/SRPMS/kernel-2.6.20-1.2320.fc5.src.rpm || die "Can't download source package."
 fi
+rpm --checksig kernel-2.6.20-1.2320.fc5.src.rpm || die "Can't verify signature."
 rpm -ivh kernel-2.6.20-1.2320.fc5.src.rpm || die "Can't install source package."
 
 cd /usr/src/redhat/SOURCES/ || die "Can't chdir to /usr/src/redhat/SOURCES/ ."
-if [ ! -r ccs-patch-1.7.2-20100401.tar.gz ]
+if [ ! -r ccs-patch-1.7.2-20100412.tar.gz ]
 then
-    wget http://sourceforge.jp/frs/redir.php?f=/tomoyo/43375/ccs-patch-1.7.2-20100401.tar.gz || die "Can't download patch."
+    wget http://sourceforge.jp/frs/redir.php?f=/tomoyo/43375/ccs-patch-1.7.2-20100412.tar.gz || die "Can't download patch."
 fi
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
@@ -60,7 +61,7 @@ patch << "EOF" || die "Can't patch spec file."
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf $RPM_SOURCE_DIR/ccs-patch-1.7.2-20100401.tar.gz
++tar -zxf $RPM_SOURCE_DIR/ccs-patch-1.7.2-20100412.tar.gz
 +# sed -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -1.2320.fc5/" -- Makefile
 +patch -sp1 < patches/ccs-patch-2.6.20-fedora-core-5.diff
 +
@@ -85,4 +86,33 @@ echo ""
 echo "Edit /tmp/ccs-kernel.spec if needed, and run"
 echo "rpmbuild -bb /tmp/ccs-kernel.spec"
 echo "to build kernel rpm packages."
+echo ""
+echo "I'll start 'rpmbuild -bb --target i586 /tmp/ccs-kernel.spec' in 30 seconds. Press Ctrl-C to stop."
+sleep 30
+patch << "EOF" || die "Can't patch spec file."
+--- /tmp/ccs-kernel.spec
++++ /tmp/ccs-kernel.spec
+@@ -3,16 +3,16 @@
+ # What parts do we want to build?  We must build at least one kernel.
+ # These are the kernels that are built IF the architecture allows it.
+ 
+-%define buildup 1
++%define buildup 0
+ # Only used on archs without run-time support (ie ppc, sparc64)
+-%define buildsmp 0
++%define buildsmp 1
+ %define buildpae 0
+ # Whether to apply the Xen patches, leave this enabled.
+ %define includexen 1
+ # Whether to build the Xen kernels, disable if you want.
+-%define buildxen 1
++%define buildxen 0
+ %define builddoc 0
+-%define buildkdump 1
++%define buildkdump 0
+ %define buildheaders 0
+ %define builddebug 0
+ 
+EOF
+exec rpmbuild -bb --target i586 /tmp/ccs-kernel.spec
 exit 0
