@@ -14,12 +14,13 @@ if [ ! -r kernel-2.4.20-46.9.legacy.src.rpm ]
 then
     wget http://ftp.riken.go.jp/Linux/fedoralegacy/redhat/9/updates/SRPMS/kernel-2.4.20-46.9.legacy.src.rpm || die "Can't download source package."
 fi
+rpm --checksig kernel-2.4.20-46.9.legacy.src.rpm || die "Can't verify signature."
 rpm -ivh kernel-2.4.20-46.9.legacy.src.rpm || die "Can't install source package."
 
 cd /usr/src/redhat/SOURCES/ || die "Can't chdir to /usr/src/redhat/SOURCES/ ."
-if [ ! -r ccs-patch-1.7.1-20091111.tar.gz ]
+if [ ! -r ccs-patch-1.7.2-20100412.tar.gz ]
 then
-    wget http://osdn.dl.sourceforge.jp/tomoyo/43375/ccs-patch-1.7.1-20091111.tar.gz || die "Can't download patch."
+    wget http://sourceforge.jp/frs/redir.php?f=/tomoyo/43375/ccs-patch-1.7.2-20100412.tar.gz || die "Can't download patch."
 fi
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
@@ -32,7 +33,7 @@ patch << "EOF" || die "Can't patch spec file."
  # adding some text to the end of the version number.
  #
 -%define release 46.9.legacy
-+%define release 46.9.legacy_tomoyo_1.7.1
++%define release 46.9.legacy_tomoyo_1.7.2
  %define sublevel 20
  %define kversion 2.4.%{sublevel}
  # /usr/src/%{kslnk} -> /usr/src/linux-%{KVERREL}
@@ -50,7 +51,7 @@ patch << "EOF" || die "Can't patch spec file."
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.7.1-20091111.tar.gz
++tar -zxf %_sourcedir/ccs-patch-1.7.2-20100412.tar.gz
 +# sed -i -e "s/^SUBLEVEL.*/SUBLEVEL = 20/" -e "s/^EXTRAVERSION.*/EXTRAVERSION = -46.9.legacycustom/" -- Makefile
 +patch -sp1 < patches/ccs-patch-2.4.20-redhat-linux-9.diff
 +
@@ -74,4 +75,26 @@ echo ""
 echo "Edit /tmp/ccs-kernel.spec if needed, and run"
 echo "rpmbuild -bb /tmp/ccs-kernel.spec"
 echo "to build kernel rpm packages."
+echo ""
+echo "I'll start 'rpmbuild -bb --target i586 /tmp/ccs-kernel.spec' in 30 seconds. Press Ctrl-C to stop."
+sleep 30
+patch << "EOF" || die "Can't patch spec file."
+--- /tmp/ccs-kernel.spec
++++ /tmp/ccs-kernel.spec
+@@ -4,10 +4,10 @@
+ # These are the kernels that are built IF the architecture allows and
+ # no contrary --with/--without arguments are given on the command line.
+ 
+-%define buildup 1
++%define buildup 0
+ %define buildsmp 1
+-%define buildBOOT 1
+-%define buildbigmem 1
++%define buildBOOT 0
++%define buildbigmem 0
+ %define buildjensen 0
+ 
+ 
+EOF
+exec rpmbuild -bb --target i586 /tmp/ccs-kernel.spec
 exit 0
