@@ -14,12 +14,13 @@ if [ ! -r kernel-2.6.27-52vl5.src.rpm ]
 then
     wget http://updates.vinelinux.org/Vine-5.0/updates/SRPMS/kernel-2.6.27-52vl5.src.rpm || die "Can't download source package."
 fi
+rpm --checksig kernel-2.6.27-52vl5.src.rpm || die "Can't verify signature."
 rpm -ivh kernel-2.6.27-52vl5.src.rpm || die "Can't install source package."
 
 cd /usr/src/vine/SOURCES/ || die "Can't chdir to /usr/src/vine/SOURCES/ ."
-if [ ! -r ccs-patch-1.7.1-20100214.tar.gz ]
+if [ ! -r ccs-patch-1.7.2-20100412.tar.gz ]
 then
-    wget http://osdn.dl.sourceforge.jp/tomoyo/43375/ccs-patch-1.7.1-20100214.tar.gz || die "Can't download patch."
+    wget http://sourceforge.jp/frs/redir.php?f=/tomoyo/43375/ccs-patch-1.7.2-20100412.tar.gz || die "Can't download patch."
 fi
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
@@ -32,7 +33,7 @@ patch << "EOF" || die "Can't patch spec file."
  %define kversion 2.6.%{sublevel}
  %define rpmversion 2.6.%{sublevel}
 -%define release 52%{?_dist_release}
-+%define release 52%{?_dist_release}_tomoyo_1.7.1p2
++%define release 52%{?_dist_release}_tomoyo_1.7.2
  
  %define make_target bzImage
  %define hdrarch %_target_cpu
@@ -60,7 +61,7 @@ patch << "EOF" || die "Can't patch spec file."
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.7.1-20100214.tar.gz
++tar -zxf %_sourcedir/ccs-patch-1.7.2-20100412.tar.gz
 +patch -sp1 < patches/ccs-patch-2.6.27-vine-linux-5.0.diff
 +
  cp %{SOURCE10} Documentation/
@@ -84,4 +85,8 @@ echo ""
 echo "Edit /tmp/ccs-kernel.spec if needed, and run"
 echo "rpmbuild -bb /tmp/ccs-kernel.spec"
 echo "to build kernel rpm packages."
+echo ""
+echo "I'll start 'rpmbuild -bb --target i586 /tmp/ccs-kernel.spec' in 30 seconds. Press Ctrl-C to stop."
+sleep 30
+exec rpmbuild -bb --target i586 /tmp/ccs-kernel.spec
 exit 0
