@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/condition.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1   2009/11/11
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -138,7 +138,7 @@ static bool ccs_scan_bprm(struct ccs_execve_entry *ee,
 		checked = local_checked;
 		memset(local_checked, 0, sizeof(local_checked));
 	} else {
-		checked = kzalloc(argc + envc, GFP_KERNEL);
+		checked = kzalloc(argc + envc, CCS_GFP_FLAGS);
 		if (!checked)
 			return false;
 	}
@@ -625,7 +625,7 @@ struct ccs_condition *ccs_get_condition(char * const condition)
 		+ names_count * sizeof(struct ccs_name_union)
 		+ argc * sizeof(struct ccs_argv_entry)
 		+ envc * sizeof(struct ccs_envp_entry);
-	entry = kzalloc(size, GFP_KERNEL);
+	entry = kzalloc(size, CCS_GFP_FLAGS);
 	if (!entry)
 		return NULL;
 	INIT_LIST_HEAD(&entry->list);
@@ -765,10 +765,10 @@ struct ccs_condition *ccs_get_condition(char * const condition)
 	BUG_ON(envc);
 	BUG_ON(condc);
 	entry->size = size;
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(ptr, &ccs_condition_list, list) {
-		if (ccs_memcmp(ptr, entry, offsetof(typeof(*entry), size),
-			       size))
+		if (!ccs_is_same_condition(ptr, entry))
 			continue;
 		/* Same entry found. Share this entry. */
 		atomic_inc(&ptr->users);
@@ -1099,10 +1099,10 @@ bool ccs_condition(struct ccs_request_info *r,
 				value = current_fsgid();
 				break;
 			case CCS_TASK_PID:
-				value = sys_getpid();
+				value = ccsecurity_exports.sys_getpid();
 				break;
 			case CCS_TASK_PPID:
-				value = sys_getppid();
+				value = ccsecurity_exports.sys_getppid();
 				break;
 			case CCS_TYPE_IS_SOCKET:
 				value = S_IFSOCK;
