@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1+   2010/02/14
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -87,7 +87,6 @@ static int ccs_update_domain_initializer_entry(const char *domainname,
 					       const bool is_not,
 					       const bool is_delete)
 {
-	struct ccs_domain_initializer_entry *entry = NULL;
 	struct ccs_domain_initializer_entry *ptr;
 	struct ccs_domain_initializer_entry e = { .is_not = is_not };
 	int error = is_delete ? -ENOENT : -ENOMEM;
@@ -106,27 +105,28 @@ static int ccs_update_domain_initializer_entry(const char *domainname,
 	e.program = ccs_get_name(program);
 	if (!e.program)
 		goto out;
-	if (!is_delete)
-		entry = kmalloc(sizeof(e), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(ptr, &ccs_domain_initializer_list, list) {
-		if (ccs_memcmp(ptr, &e, offsetof(typeof(e), is_not),
-			       sizeof(e)))
+		if (!ccs_is_same_domain_initializer_entry(ptr, &e))
 			continue;
 		ptr->is_deleted = is_delete;
 		error = 0;
 		break;
 	}
-	if (!is_delete && error && ccs_commit_ok(entry, &e, sizeof(e))) {
-		list_add_tail_rcu(&entry->list, &ccs_domain_initializer_list);
-		entry = NULL;
-		error = 0;
+	if (!is_delete && error) {
+		struct ccs_domain_initializer_entry *entry =
+			ccs_commit_ok(&e, sizeof(e));
+		if (entry) {
+			list_add_tail_rcu(&entry->list,
+					  &ccs_domain_initializer_list);
+			error = 0;
+		}
 	}
 	mutex_unlock(&ccs_policy_lock);
  out:
 	ccs_put_name(e.domainname);
 	ccs_put_name(e.program);
-	kfree(entry);
 	return error;
 }
 
@@ -248,7 +248,6 @@ static int ccs_update_domain_keeper_entry(const char *domainname,
 					  const bool is_not,
 					  const bool is_delete)
 {
-	struct ccs_domain_keeper_entry *entry = NULL;
 	struct ccs_domain_keeper_entry *ptr;
 	struct ccs_domain_keeper_entry e = { .is_not = is_not };
 	int error = is_delete ? -ENOENT : -ENOMEM;
@@ -267,27 +266,28 @@ static int ccs_update_domain_keeper_entry(const char *domainname,
 	e.domainname = ccs_get_name(domainname);
 	if (!e.domainname)
 		goto out;
-	if (!is_delete)
-		entry = kmalloc(sizeof(e), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(ptr, &ccs_domain_keeper_list, list) {
-		if (ccs_memcmp(ptr, &e, offsetof(typeof(e), is_not),
-			       sizeof(e)))
+		if (!ccs_is_same_domain_keeper_entry(ptr, &e))
 			continue;
 		ptr->is_deleted = is_delete;
 		error = 0;
 		break;
 	}
-	if (!is_delete && error && ccs_commit_ok(entry, &e, sizeof(e))) {
-		list_add_tail_rcu(&entry->list, &ccs_domain_keeper_list);
-		entry = NULL;
-		error = 0;
+	if (!is_delete && error) {
+		struct ccs_domain_keeper_entry *entry =
+			ccs_commit_ok(&e, sizeof(e));
+		if (entry) {
+			list_add_tail_rcu(&entry->list,
+					  &ccs_domain_keeper_list);
+			error = 0;
+		}
 	}
 	mutex_unlock(&ccs_policy_lock);
  out:
 	ccs_put_name(e.domainname);
 	ccs_put_name(e.program);
-	kfree(entry);
 	return error;
 }
 
@@ -402,7 +402,6 @@ static int ccs_update_aggregator_entry(const char *original_name,
 				       const char *aggregated_name,
 				       const bool is_delete)
 {
-	struct ccs_aggregator_entry *entry = NULL;
 	struct ccs_aggregator_entry *ptr;
 	struct ccs_aggregator_entry e = { };
 	int error = is_delete ? -ENOENT : -ENOMEM;
@@ -413,27 +412,27 @@ static int ccs_update_aggregator_entry(const char *original_name,
 	e.aggregated_name = ccs_get_name(aggregated_name);
 	if (!e.original_name || !e.aggregated_name)
 		goto out;
-	if (!is_delete)
-		entry = kmalloc(sizeof(e), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(ptr, &ccs_aggregator_list, list) {
-		if (ccs_memcmp(ptr, &e, offsetof(typeof(e), original_name),
-			       sizeof(e)))
+		if (!ccs_is_same_aggregator_entry(ptr, &e))
 			continue;
 		ptr->is_deleted = is_delete;
 		error = 0;
 		break;
 	}
-	if (!is_delete && error && ccs_commit_ok(entry, &e, sizeof(e))) {
-		list_add_tail_rcu(&entry->list, &ccs_aggregator_list);
-		entry = NULL;
-		error = 0;
+	if (!is_delete && error) {
+		struct ccs_aggregator_entry *entry =
+			ccs_commit_ok(&e, sizeof(e));
+		if (entry) {
+			list_add_tail_rcu(&entry->list, &ccs_aggregator_list);
+			error = 0;
+		}
 	}
 	mutex_unlock(&ccs_policy_lock);
  out:
 	ccs_put_name(e.original_name);
 	ccs_put_name(e.aggregated_name);
-	kfree(entry);
 	return error;
 }
 
@@ -495,7 +494,8 @@ int ccs_delete_domain(char *domainname)
 	struct ccs_path_info name;
 	name.name = domainname;
 	ccs_fill_path_info(&name);
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		return 0;
 	/* Is there an active domain? */
 	list_for_each_entry_rcu(domain, &ccs_domain_list, list) {
 		/* Never delete ccs_kernel_domain */
@@ -523,7 +523,7 @@ struct ccs_domain_info *ccs_find_or_assign_new_domain(const char *domainname,
 						      const u8 profile)
 {
 	struct ccs_domain_info *entry;
-	struct ccs_domain_info *domain;
+	struct ccs_domain_info *domain = NULL;
 	const struct ccs_path_info *saved_domainname;
 	bool found = false;
 
@@ -532,8 +532,9 @@ struct ccs_domain_info *ccs_find_or_assign_new_domain(const char *domainname,
 	saved_domainname = ccs_get_name(domainname);
 	if (!saved_domainname)
 		return NULL;
-	entry = kzalloc(sizeof(*entry), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	entry = kzalloc(sizeof(*entry), CCS_GFP_FLAGS);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(domain, &ccs_domain_list, list) {
 		if (domain->is_deleted ||
 		    ccs_pathcmp(saved_domainname, domain->domainname))
@@ -552,6 +553,7 @@ struct ccs_domain_info *ccs_find_or_assign_new_domain(const char *domainname,
 		found = true;
 	}
 	mutex_unlock(&ccs_policy_lock);
+ out:
 	ccs_put_name(saved_domainname);
 	kfree(entry);
 	return found ? domain : NULL;
@@ -621,7 +623,7 @@ static int ccs_find_next_domain(struct ccs_execve_entry *ee)
 
 		/* Check execute permission. */
 		retval = ccs_exec_perm(r, &rn);
-		if (retval == 1)
+		if (retval == CCS_RETRY_REQUEST)
 			goto retry;
 		if (retval < 0)
 			goto out;
@@ -655,7 +657,7 @@ static int ccs_find_next_domain(struct ccs_execve_entry *ee)
 	if (r->mode == CCS_CONFIG_ENFORCING) {
 		int error = ccs_supervisor(r, "# wants to create domain\n"
 					   "%s\n", ee->tmp);
-		if (error == 1)
+		if (error == CCS_RETRY_REQUEST)
 			goto retry;
 		if (error < 0)
 			goto done;
@@ -708,7 +710,7 @@ static int ccs_environ(struct ccs_execve_entry *ee)
 	int error = -ENOMEM;
 	if (!r->mode || !envp_count)
 		return 0;
-	arg_ptr = kzalloc(CCS_EXEC_TMPSIZE, GFP_KERNEL);
+	arg_ptr = kzalloc(CCS_EXEC_TMPSIZE, CCS_GFP_FLAGS);
 	if (!arg_ptr)
 		goto out;
 	while (error == -ENOMEM) {
@@ -970,9 +972,10 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 			 "pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d "
 			 "sgid=%d fsuid=%d fsgid=%d state[0]=%u "
 			 "state[1]=%u state[2]=%u",
-			 (pid_t) sys_getpid(), current_uid(), current_gid(),
-			 current_euid(), current_egid(), current_suid(),
-			 current_sgid(), current_fsuid(), current_fsgid(),
+			 (pid_t) ccsecurity_exports.sys_getpid(),
+			 current_uid(), current_gid(), current_euid(),
+			 current_egid(), current_suid(), current_sgid(),
+			 current_fsuid(), current_fsgid(),
 			 (u8) (ccs_flags >> 24), (u8) (ccs_flags >> 16),
 			 (u8) (ccs_flags >> 8));
 		retval = copy_strings_kernel(1, &cp, bprm);
@@ -1012,7 +1015,7 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 	{
 		int depth = ccs_get_root_depth();
 		int len = ee->handler->total_len + 1;
-		char *cp = kmalloc(len, GFP_KERNEL);
+		char *cp = kmalloc(len, CCS_GFP_FLAGS);
 		if (!cp) {
 			retval = -ENOMEM;
 			goto out;
@@ -1117,7 +1120,7 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 	struct page *page;
 	/* dump->data is released by ccs_finish_execve(). */
 	if (!dump->data) {
-		dump->data = kzalloc(PAGE_SIZE, GFP_KERNEL);
+		dump->data = kzalloc(PAGE_SIZE, CCS_GFP_FLAGS);
 		if (!dump->data)
 			return false;
 	}
@@ -1165,18 +1168,17 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
  *
  * Returns 0 on success, negative value otherwise.
  */
-int ccs_start_execve(struct linux_binprm *bprm, struct ccs_execve_entry **eep)
+static int ccs_start_execve(struct linux_binprm *bprm,
+			    struct ccs_execve_entry **eep)
 {
 	int retval;
 	struct task_struct *task = current;
 	struct ccs_execve_entry *ee;
 	*eep = NULL;
-	if (!ccs_policy_loaded)
-		ccs_load_policy(bprm->filename);
-	ee = kzalloc(sizeof(*ee), GFP_KERNEL);
+	ee = kzalloc(sizeof(*ee), CCS_GFP_FLAGS);
 	if (!ee)
 		return -ENOMEM;
-	ee->tmp = kzalloc(CCS_EXEC_TMPSIZE, GFP_KERNEL);
+	ee->tmp = kzalloc(CCS_EXEC_TMPSIZE, CCS_GFP_FLAGS);
 	if (!ee->tmp) {
 		kfree(ee);
 		return -ENOMEM;
@@ -1244,7 +1246,7 @@ int ccs_start_execve(struct linux_binprm *bprm, struct ccs_execve_entry **eep)
  *
  * Caller holds ccs_read_lock().
  */
-void ccs_finish_execve(int retval, struct ccs_execve_entry *ee)
+static void ccs_finish_execve(int retval, struct ccs_execve_entry *ee)
 {
 	struct task_struct *task = current;
 	if (!ee)
@@ -1272,3 +1274,62 @@ void ccs_finish_execve(int retval, struct ccs_execve_entry *ee)
 	kfree(ee->dump.data);
 	kfree(ee);
 }
+
+/**
+ * ccs_may_transit - Check permission and do domain transition without execve().
+ *
+ * @domainname: Domainname to transit to.
+ * @pathname: Pathname to check.
+ *
+ * Returns 0 on success, negative value otherwise.
+ *
+ * Caller holds ccs_read_lock().
+ */
+int ccs_may_transit(const char *domainname, const char *pathname)
+{
+	struct ccs_path_info name;
+	struct ccs_request_info r;
+	struct ccs_domain_info *domain;
+	int error;
+	name.name = pathname;
+	ccs_fill_path_info(&name);
+	/* Check allow_transit permission. */
+	ccs_init_request_info(&r, NULL, CCS_MAC_FILE_TRANSIT);
+	error = ccs_path_permission(&r, CCS_TYPE_TRANSIT, &name);
+	if (error)
+		return error;
+	/* Check destination domain. */
+	domain = ccs_find_domain(domainname);
+	if (!domain && r.mode != CCS_CONFIG_ENFORCING &&
+	    strlen(domainname) < CCS_EXEC_TMPSIZE - 10) {
+		domain = ccs_find_or_assign_new_domain(domainname, r.profile);
+		if (domain)
+			ccs_audit_domain_creation_log(domain);
+	}
+	if (domain) {
+		error = 0;
+		current->ccs_domain_info = domain;
+	} else {
+		error = -ENOENT;
+	}
+	return error;
+}
+
+static int __ccs_search_binary_handler(struct linux_binprm *bprm,
+				       struct pt_regs *regs)
+{
+	struct ccs_execve_entry *ee;
+	int retval;
+	if (!ccs_policy_loaded)
+		ccsecurity_exports.load_policy(bprm->filename);
+	retval = ccs_start_execve(bprm, &ee);
+	if (!retval)
+		retval = search_binary_handler(bprm, regs);
+	ccs_finish_execve(retval, ee);
+	return retval;
+}
+
+void __init ccs_domain_init(void)
+{
+	ccsecurity_ops.search_binary_handler = __ccs_search_binary_handler;
+}
