@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/mount.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1   2009/11/11
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -85,23 +85,6 @@ static int ccs_audit_mount_log(struct ccs_request_info *r,
 				   type, flags);
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-/* For compatibility with older kernels. */
-static inline void module_put(struct module *module)
-{
-	if (module)
-		__MOD_DEC_USE_COUNT(module);
-}
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
-/* For compatibility with older kernels. */
-static void put_filesystem(struct file_system_type *fs)
-{
-	module_put(fs->owner);
-}
-#endif
-
 /**
  * ccs_mount_acl2 - Check permission for mount() operation.
  *
@@ -198,8 +181,7 @@ static int ccs_mount_acl2(struct ccs_request_info *r, char *dev_name,
 		struct ccs_mount_acl *acl;
 		if (ptr->is_deleted || ptr->type != CCS_TYPE_MOUNT_ACL)
 			continue;
-		acl = container_of(ptr, struct ccs_mount_acl,
-				   head);
+		acl = container_of(ptr, struct ccs_mount_acl, head);
 		if (!ccs_compare_number_union(flags, &acl->flags) ||
 		    !ccs_compare_name_union(&rtype, &acl->fs_type) ||
 		    !ccs_compare_name_union(&rdir, &acl->dir_name) ||
@@ -223,7 +205,7 @@ static int ccs_mount_acl2(struct ccs_request_info *r, char *dev_name,
 	kfree(requested_dev_name);
 	kfree(requested_dir_name);
 	if (fstype)
-		put_filesystem(fstype);
+		ccsecurity_exports.put_filesystem(fstype);
 	kfree(requested_type);
 	/* Drop refcount obtained by ccs_get_path(). */
 	if (obj.path1.dentry)
@@ -312,38 +294,27 @@ static int ccs_mount_acl(struct ccs_request_info *r, char *dev_name,
 	else
 		do {
 			error = ccs_mount_acl2(r, dev_name, dir, type, flags);
-		} while (error == 1);
+		} while (error == CCS_RETRY_REQUEST);
 	if (r->mode != CCS_CONFIG_ENFORCING)
 		error = 0;
 	return error;
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
-#define PATH_or_NAMEIDATA path
-#else
-#define PATH_or_NAMEIDATA nameidata
-#endif
-
 /**
  * ccs_mount_permission - Check permission for mount() operation.
  *
  * @dev_name:  Name of device file.
- * @path:      Pointer to "struct path" (for 2.6.26 and later).
- *             Pointer to "struct nameidata" (for 2.6.25 and earlier).
+ * @path:      Pointer to "struct path".
  * @type:      Name of filesystem type. May be NULL.
  * @flags:     Mount options.
  * @data_page: Optional data. May be NULL.
  *
  * Returns 0 on success, negative value otherwise.
  */
-int ccs_mount_permission(char *dev_name, struct PATH_or_NAMEIDATA *path,
-			 char *type, unsigned long flags, void *data_page)
+static int __ccs_mount_permission(char *dev_name, struct path *path,
+				  char *type, unsigned long flags,
+				  void *data_page)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
-	struct path dir = { path->mnt, path->dentry };
-#elif LINUX_VERSION_CODE == KERNEL_VERSION(2, 6, 25)
-	struct path dir = { path->path.mnt, path->path.dentry };
-#endif
 	struct ccs_request_info r;
 	int error;
 	int idx;
@@ -355,11 +326,7 @@ int ccs_mount_permission(char *dev_name, struct PATH_or_NAMEIDATA *path,
 	if (!type)
 		type = "<NULL>";
 	idx = ccs_read_lock();
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
 	error = ccs_mount_acl(&r, dev_name, path, type, flags);
-#else
-	error = ccs_mount_acl(&r, dev_name, &dir, type, flags);
-#endif
 	ccs_read_unlock(idx);
 	return error;
 }
@@ -378,7 +345,6 @@ int ccs_write_mount_policy(char *data, struct ccs_domain_info *domain,
 			   struct ccs_condition *condition,
 			   const bool is_delete)
 {
-	struct ccs_mount_acl *entry = NULL;
 	struct ccs_acl_info *ptr;
 	struct ccs_mount_acl e = { .head.type = CCS_TYPE_MOUNT_ACL,
 				   .head.cond = condition };
@@ -391,24 +357,23 @@ int ccs_write_mount_policy(char *data, struct ccs_domain_info *domain,
 	    !ccs_parse_name_union(w[2], &e.fs_type) ||
 	    !ccs_parse_number_union(w[3], &e.flags))
 		goto out;
-	if (!is_delete)
-		entry = kmalloc(sizeof(e), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(ptr, &domain->acl_info_list, list) {
 		struct ccs_mount_acl *acl =
 			container_of(ptr, struct ccs_mount_acl, head);
-		if (ptr->type != CCS_TYPE_MOUNT_ACL || ptr->cond != condition
-		    || ccs_memcmp(acl, &e, offsetof(typeof(e), dev_name),
-				  sizeof(e)))
+		if (!ccs_is_same_mount_acl(acl, &e))
 			continue;
 		ptr->is_deleted = is_delete;
 		error = 0;
 		break;
 	}
-	if (!is_delete && error && ccs_commit_ok(entry, &e, sizeof(e))) {
-		ccs_add_domain_acl(domain, &entry->head);
-		entry = NULL;
-		error = 0;
+	if (!is_delete && error) {
+		struct ccs_mount_acl *entry = ccs_commit_ok(&e, sizeof(e));
+		if (entry) {
+			ccs_add_domain_acl(domain, &entry->head);
+			error = 0;
+		}
 	}
 	mutex_unlock(&ccs_policy_lock);
  out:
@@ -416,6 +381,24 @@ int ccs_write_mount_policy(char *data, struct ccs_domain_info *domain,
 	ccs_put_name_union(&e.dir_name);
 	ccs_put_name_union(&e.fs_type);
 	ccs_put_number_union(&e.flags);
-	kfree(entry);
 	return error;
 }
+
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 24)
+static int ccs_old_mount_permission(char *dev_name, struct nameidata *nd,
+				    char *type, unsigned long flags,
+				    void *data_page)
+{
+	struct path path = { nd->mnt, nd->dentry };
+	return __ccs_mount_permission(dev_name, &path, type, flags, data_page);
+}
+#endif
+
+void __init ccs_mount_init(void)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+	ccsecurity_ops.mount_permission = __ccs_mount_permission;
+#else
+	ccsecurity_ops.mount_permission = ccs_old_mount_permission;
+#endif
+}
