@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/policy_io.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.0   2009/09/04
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -16,11 +16,13 @@ static struct ccs_profile ccs_default_profile = {
 	.learning = &ccs_default_profile.preference,
 	.permissive = &ccs_default_profile.preference,
 	.enforcing = &ccs_default_profile.preference,
-#ifdef CONFIG_CCSECURITY_AUDIT
 	.audit = &ccs_default_profile.preference,
+#ifdef CONFIG_CCSECURITY_AUDIT
 	.preference.audit_max_grant_log = CONFIG_CCSECURITY_MAX_GRANT_LOG,
 	.preference.audit_max_reject_log = CONFIG_CCSECURITY_MAX_REJECT_LOG,
 #endif
+	.preference.audit_task_info = true,
+	.preference.audit_path_info = true,
 	.preference.enforcing_penalty = 0,
 	.preference.enforcing_verbose = true,
 	.preference.learning_max_entry = CONFIG_CCSECURITY_MAX_ACCEPT_ENTRY,
@@ -31,12 +33,12 @@ static struct ccs_profile ccs_default_profile = {
 	.preference.permissive_verbose = true
 };
 
+/* Profile version. Currently only 20090903 is defined. */
+static unsigned int ccs_profile_version;
+
 /* Profile table. Memory is allocated as needed. */
 static struct ccs_profile *ccs_profile_ptr[CCS_MAX_PROFILES];
 
-/* Lock for protecting "struct ccs_profile"->comment  */
-static DEFINE_SPINLOCK(ccs_profile_comment_lock);
-
 /* String table for functionality that takes 4 modes. */
 static const char *ccs_mode_4[4] = {
 	"disabled", "learning", "permissive", "enforcing"
@@ -92,6 +94,8 @@ static const char *ccs_mac_keywords[CCS_MAX_MAC_INDEX +
 	= "file::umount",
 	[CCS_MAC_FILE_PIVOT_ROOT]
 	= "file::pivot_root",
+	[CCS_MAC_FILE_TRANSIT]
+	= "file::transit",
 	[CCS_MAC_ENVIRON]
 	= "misc::env",
 	[CCS_MAC_NETWORK_UDP_BIND]
@@ -258,14 +262,13 @@ static struct ccs_profile *ccs_find_or_assign_new_profile(const unsigned int
 	ptr = ccs_profile_ptr[profile];
 	if (ptr)
 		return ptr;
-	entry = kzalloc(sizeof(*entry), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	entry = kzalloc(sizeof(*entry), CCS_GFP_FLAGS);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	ptr = ccs_profile_ptr[profile];
 	if (!ptr && ccs_memory_ok(entry, sizeof(*entry))) {
 		ptr = entry;
-#ifdef CONFIG_CCSECURITY_AUDIT
 		ptr->audit = &ccs_default_profile.preference;
-#endif
 		ptr->learning = &ccs_default_profile.preference;
 		ptr->permissive = &ccs_default_profile.preference;
 		ptr->enforcing = &ccs_default_profile.preference;
@@ -278,6 +281,7 @@ static struct ccs_profile *ccs_find_or_assign_new_profile(const unsigned int
 		entry = NULL;
 	}
 	mutex_unlock(&ccs_policy_lock);
+ out:
 	kfree(entry);
 	return ptr;
 }
@@ -285,9 +289,10 @@ static struct ccs_profile *ccs_find_or_assign_new_profile(const unsigned int
 /**
  * ccs_check_profile - Check all profiles currently assigned to domains are defined.
  */
-void ccs_check_profile(void)
+static void ccs_check_profile(void)
 {
 	struct ccs_domain_info *domain;
+	const int idx = ccs_read_lock();
 	ccs_policy_loaded = true;
 	list_for_each_entry_rcu(domain, &ccs_domain_list, list) {
 		const u8 profile = domain->profile;
@@ -296,6 +301,12 @@ void ccs_check_profile(void)
 		panic("Profile %u (used by '%s') not defined.\n",
 		      profile, domain->domainname->name);
 	}
+	ccs_read_unlock(idx);
+	if (ccs_profile_version != 20090903)
+		panic("Profile version %u is not supported.\n",
+		      ccs_profile_version);
+	printk(KERN_INFO "CCSecurity: 1.7.2   2010/04/01\n");
+	printk(KERN_INFO "Mandatory Access Control activated.\n");
 }
 
 /**
@@ -331,6 +342,8 @@ static int ccs_write_profile(struct ccs_io_buffer *head)
 	bool use_default = false;
 	char *cp;
 	struct ccs_profile *profile;
+	if (sscanf(data, "PROFILE_VERSION=%u", &ccs_profile_version) == 1)
+		return 0;
 	i = simple_strtoul(data, &cp, 10);
 	if (data == cp) {
 		profile = &ccs_default_profile;
@@ -354,14 +367,16 @@ static int ccs_write_profile(struct ccs_io_buffer *head)
 		value = 0;
 	else
 		value = -1;
-#ifdef CONFIG_CCSECURITY_AUDIT
 	if (!strcmp(data, "PREFERENCE::audit")) {
+#ifdef CONFIG_CCSECURITY_AUDIT
 		char *cp2;
+#endif
 		if (use_default) {
 			profile->audit = &ccs_default_profile.preference;
 			return 0;
 		}
 		profile->audit = &profile->preference;
+#ifdef CONFIG_CCSECURITY_AUDIT
 		cp2 = strstr(cp, "max_grant_log=");
 		if (cp2)
 			sscanf(cp2 + 14, "%u",
@@ -370,9 +385,17 @@ static int ccs_write_profile(struct ccs_io_buffer *head)
 		if (cp2)
 			sscanf(cp2 + 15, "%u",
 			       &profile->preference.audit_max_reject_log);
+#endif
+		if (strstr(cp, "task_info=yes"))
+			profile->preference.audit_task_info = true;
+		else if (strstr(cp, "task_info=no"))
+			profile->preference.audit_task_info = false;
+		if (strstr(cp, "path_info=yes"))
+			profile->preference.audit_path_info = true;
+		else if (strstr(cp, "path_info=no"))
+			profile->preference.audit_path_info = false;
 		return 0;
 	}
-#endif
 	if (!strcmp(data, "PREFERENCE::enforcing")) {
 		char *cp2;
 		if (use_default) {
@@ -428,13 +451,8 @@ static int ccs_write_profile(struct ccs_io_buffer *head)
 	if (profile == &ccs_default_profile)
 		return -EINVAL;
 	if (!strcmp(data, "COMMENT")) {
-		const struct ccs_path_info *new_comment = ccs_get_name(cp);
-		const struct ccs_path_info *old_comment;
-		/* Protect reader from ccs_put_name(). */
-		spin_lock(&ccs_profile_comment_lock);
-		old_comment = profile->comment;
-		profile->comment = new_comment;
-		spin_unlock(&ccs_profile_comment_lock);
+		const struct ccs_path_info *old_comment = profile->comment;
+		profile->comment = ccs_get_name(cp);
 		ccs_put_name(old_comment);
 		return 0;
 	}
@@ -501,12 +519,19 @@ static void ccs_read_profile(struct ccs_io_buffer *head)
 	if (head->read_bit)
 		goto body;
 	ccs_io_printf(head, "PROFILE_VERSION=%s\n", "20090903");
+	ccs_io_printf(head, "PREFERENCE::audit={ "
 #ifdef CONFIG_CCSECURITY_AUDIT
-	ccs_io_printf(head, "PREFERENCE::audit={ max_grant_log=%u "
-		      "max_reject_log=%u }\n",
-		      ccs_default_profile.preference.audit_max_grant_log,
-		      ccs_default_profile.preference.audit_max_reject_log);
+		      "max_grant_log=%u max_reject_log=%u "
 #endif
+		      "task_info=%s path_info=%s }\n",
+#ifdef CONFIG_CCSECURITY_AUDIT
+		      ccs_default_profile.preference.audit_max_grant_log,
+		      ccs_default_profile.preference.audit_max_reject_log,
+#endif
+		      ccs_yesno(ccs_default_profile.preference.
+				audit_task_info),
+		      ccs_yesno(ccs_default_profile.preference.
+				audit_path_info));
 	ccs_io_printf(head, "PREFERENCE::learning={ verbose=%s max_entry=%u "
 		      "exec.realpath=%s exec.argv0=%s symlink.target=%s }\n",
 		      ccs_yesno(ccs_default_profile.preference.
@@ -534,15 +559,14 @@ static void ccs_read_profile(struct ccs_io_buffer *head)
 		int i;
 		int pos;
 		const struct ccs_profile *profile = ccs_profile_ptr[index];
+		const struct ccs_path_info *comment;
 		head->read_step = index;
 		if (!profile)
 			continue;
 		pos = head->read_avail;
-		spin_lock(&ccs_profile_comment_lock);
+		comment = profile->comment;
 		done = ccs_io_printf(head, "%u-COMMENT=%s\n", index,
-				     profile->comment ? profile->comment->name
-				     : "");
-		spin_unlock(&ccs_profile_comment_lock);
+				     comment ? comment->name : "");
 		if (!done)
 			goto out;
 		config = profile->default_config;
@@ -584,15 +608,21 @@ static void ccs_read_profile(struct ccs_io_buffer *head)
 				goto out;
 #endif
 		}
-#ifdef CONFIG_CCSECURITY_AUDIT
 		if (profile->audit != &ccs_default_profile.preference &&
 		    !ccs_io_printf(head, "%u-PREFERENCE::audit={ "
-				   "max_grant_log=%u max_reject_log=%u }\n",
-				   index,
-				   profile->preference.audit_max_grant_log,
-				   profile->preference.audit_max_reject_log))
-			goto out;
+#ifdef CONFIG_CCSECURITY_AUDIT
+				   "max_grant_log=%u max_reject_log=%u "
 #endif
+				   "task_info=%s path_info=%s }\n", index,
+#ifdef CONFIG_CCSECURITY_AUDIT
+				   profile->preference.audit_max_grant_log,
+				   profile->preference.audit_max_reject_log,
+#endif
+				   ccs_yesno(profile->preference.
+					     audit_task_info),
+				   ccs_yesno(profile->preference.
+					     audit_path_info)))
+			goto out;
 		if (profile->learning != &ccs_default_profile.preference &&
 		    !ccs_io_printf(head, "%u-PREFERENCE::learning={ "
 				   "verbose=%s max_entry=%u exec.realpath=%s "
@@ -643,7 +673,6 @@ LIST_HEAD(ccs_policy_manager_list);
  */
 static int ccs_update_manager_entry(const char *manager, const bool is_delete)
 {
-	struct ccs_policy_manager_entry *entry = NULL;
 	struct ccs_policy_manager_entry *ptr;
 	struct ccs_policy_manager_entry e = { };
 	int error = is_delete ? -ENOENT : -ENOMEM;
@@ -658,9 +687,8 @@ static int ccs_update_manager_entry(const char *manager, const bool is_delete)
 	e.manager = ccs_get_name(manager);
 	if (!e.manager)
 		return -ENOMEM;
-	if (!is_delete)
-		entry = kmalloc(sizeof(e), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(ptr, &ccs_policy_manager_list, list) {
 		if (ptr->manager != e.manager)
 			continue;
@@ -668,14 +696,18 @@ static int ccs_update_manager_entry(const char *manager, const bool is_delete)
 		error = 0;
 		break;
 	}
-	if (!is_delete && error && ccs_commit_ok(entry, &e, sizeof(e))) {
-		list_add_tail_rcu(&entry->list, &ccs_policy_manager_list);
-		entry = NULL;
-		error = 0;
+	if (!is_delete && error) {
+		struct ccs_policy_manager_entry *entry =
+			ccs_commit_ok(&e, sizeof(e));
+		if (entry) {
+			list_add_tail_rcu(&entry->list,
+					  &ccs_policy_manager_list);
+			error = 0;
+		}
 	}
 	mutex_unlock(&ccs_policy_lock);
+ out:
 	ccs_put_name(e.manager);
-	kfree(entry);
 	return error;
 }
 
@@ -824,18 +856,19 @@ static bool ccs_is_select_one(struct ccs_io_buffer *head, const char *data)
 	if (sscanf(data, "pid=%u", &pid) == 1 ||
 	    (global_pid = true, sscanf(data, "global-pid=%u", &pid) == 1)) {
 		struct task_struct *p;
-		read_lock(&tasklist_lock);
+		ccs_tasklist_lock();
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
 		if (global_pid)
-			p = find_task_by_pid_ns(pid, &init_pid_ns);
+			p = ccsecurity_exports.find_task_by_pid_ns(pid,
+							       &init_pid_ns);
 		else
-			p = find_task_by_vpid(pid);
+			p = ccsecurity_exports.find_task_by_vpid(pid);
 #else
 		p = find_task_by_pid(pid);
 #endif
 		if (p)
 			domain = ccs_task_domain(p);
-		read_unlock(&tasklist_lock);
+		ccs_tasklist_unlock();
 	} else if (!strncmp(data, "domain=", 7)) {
 		if (ccs_is_domain_def(data + 7))
 			domain = ccs_find_domain(data + 7);
@@ -881,13 +914,6 @@ static int ccs_write_domain_policy2(char *data, struct ccs_domain_info *domain,
 		return ccs_write_env_policy(data, domain, cond, is_delete);
 	if (ccs_str_starts(&data, CCS_KEYWORD_ALLOW_MOUNT))
 		return ccs_write_mount_policy(data, domain, cond, is_delete);
-	if (ccs_str_starts(&data, CCS_KEYWORD_ALLOW_UNMOUNT))
-		return ccs_write_umount_policy(data, domain, cond, is_delete);
-	if (ccs_str_starts(&data, CCS_KEYWORD_ALLOW_CHROOT))
-		return ccs_write_chroot_policy(data, domain, cond, is_delete);
-	if (ccs_str_starts(&data, CCS_KEYWORD_ALLOW_PIVOT_ROOT))
-		return ccs_write_pivot_root_policy(data, domain, cond,
-						   is_delete);
 	return ccs_write_file_policy(data, domain, cond, is_delete);
 }
 
@@ -945,6 +971,14 @@ static int ccs_write_domain_policy(struct ccs_io_buffer *head)
 		domain->ignore_global_allow_env = !is_delete;
 		return 0;
 	}
+	if (!strcmp(data, CCS_KEYWORD_QUOTA_EXCEEDED)) {
+		domain->quota_warned = !is_delete;
+		return 0;
+	}
+	if (!strcmp(data, CCS_KEYWORD_TRANSITION_FAILED)) {
+		domain->domain_transition_failed = !is_delete;
+		return 0;
+	}
 	cp = ccs_find_condition_part(data);
 	if (cp) {
 		cond = ccs_get_condition(cp);
@@ -1186,7 +1220,7 @@ static bool ccs_print_condition(struct ccs_io_buffer *head,
 }
 
 /**
- * ccs_print_path_acl - Print a single path ACL entry.
+ * ccs_print_path_acl - Print a path ACL entry.
  *
  * @head: Pointer to "struct ccs_io_buffer".
  * @ptr:  Pointer to "struct ccs_path_acl".
@@ -1204,7 +1238,8 @@ static bool ccs_print_path_acl(struct ccs_io_buffer *head,
 	for (bit = head->read_bit; bit < CCS_MAX_PATH_OPERATION; bit++) {
 		if (!(perm & (1 << bit)))
 			continue;
-		if (head->read_execute_only && bit != CCS_TYPE_EXECUTE)
+		if (head->read_execute_only && bit != CCS_TYPE_EXECUTE
+		    && bit != CCS_TYPE_TRANSIT)
 			continue;
 		/* Print "read/write" instead of "read" and "write". */
 		if ((bit == CCS_TYPE_READ || bit == CCS_TYPE_WRITE)
@@ -1536,76 +1571,6 @@ static bool ccs_print_mount_acl(struct ccs_io_buffer *head,
 	return true;
 }
 
-/**
- * ccs_print_umount_acl - Print a mount ACL entry.
- *
- * @head: Pointer to "struct ccs_io_buffer".
- * @ptr:  Pointer to "struct ccs_umount_acl".
- * @cond: Pointer to "struct ccs_condition". May be NULL.
- *
- * Returns true on success, false otherwise.
- */
-static bool ccs_print_umount_acl(struct ccs_io_buffer *head,
-				 struct ccs_umount_acl *ptr,
-				 const struct ccs_condition *cond)
-{
-	const int pos = head->read_avail;
-	if (!ccs_io_printf(head, CCS_KEYWORD_ALLOW_UNMOUNT) ||
-	    !ccs_print_name_union(head, &ptr->dir) ||
-	    !ccs_print_condition(head, cond)) {
-		head->read_avail = pos;
-		return false;
-	}
-	return true;
-}
-
-/**
- * ccs_print_chroot_acl - Print a chroot ACL entry.
- *
- * @head: Pointer to "struct ccs_io_buffer".
- * @ptr:  Pointer to "struct ccs_chroot_acl".
- * @cond: Pointer to "struct ccs_condition". May be NULL.
- *
- * Returns true on success, false otherwise.
- */
-static bool ccs_print_chroot_acl(struct ccs_io_buffer *head,
-				 struct ccs_chroot_acl *ptr,
-				 const struct ccs_condition *cond)
-{
-	const int pos = head->read_avail;
-	if (!ccs_io_printf(head, CCS_KEYWORD_ALLOW_CHROOT) ||
-	    !ccs_print_name_union(head, &ptr->dir) ||
-	    !ccs_print_condition(head, cond)) {
-		head->read_avail = pos;
-		return false;
-	}
-	return true;
-}
-
-/**
- * ccs_print_pivot_root_acl - Print a pivot_root ACL entry.
- *
- * @head: Pointer to "struct ccs_io_buffer".
- * @ptr:  Pointer to "struct ccs_pivot_root_acl".
- * @cond: Pointer to "struct ccs_condition". May be NULL.
- *
- * Returns true on success, false otherwise.
- */
-static bool ccs_print_pivot_root_acl(struct ccs_io_buffer *head,
-				     struct ccs_pivot_root_acl *ptr,
-				     const struct ccs_condition *cond)
-{
-	const int pos = head->read_avail;
-	if (!ccs_io_printf(head, CCS_KEYWORD_ALLOW_PIVOT_ROOT) ||
-	    !ccs_print_name_union(head, &ptr->new_root) ||
-	    !ccs_print_name_union(head, &ptr->old_root) ||
-	    !ccs_print_condition(head, cond)) {
-		head->read_avail = pos;
-		return false;
-	}
-	return true;
-}
-
 /**
  * ccs_print_entry - Print an ACL entry.
  *
@@ -1623,8 +1588,7 @@ static bool ccs_print_entry(struct ccs_io_buffer *head,
 		return true;
 	if (acl_type == CCS_TYPE_PATH_ACL) {
 		struct ccs_path_acl *acl
-			= container_of(ptr, struct ccs_path_acl,
-				       head);
+			= container_of(ptr, struct ccs_path_acl, head);
 		return ccs_print_path_acl(head, acl, cond);
 	}
 	if (acl_type == CCS_TYPE_EXECUTE_HANDLER) {
@@ -1650,14 +1614,12 @@ static bool ccs_print_entry(struct ccs_io_buffer *head,
 	}
 	if (acl_type == CCS_TYPE_PATH2_ACL) {
 		struct ccs_path2_acl *acl
-			= container_of(ptr, struct ccs_path2_acl,
-				       head);
+			= container_of(ptr, struct ccs_path2_acl, head);
 		return ccs_print_path2_acl(head, acl, cond);
 	}
 	if (acl_type == CCS_TYPE_PATH_NUMBER_ACL) {
 		struct ccs_path_number_acl *acl
-			= container_of(ptr, struct ccs_path_number_acl,
-				       head);
+			= container_of(ptr, struct ccs_path_number_acl, head);
 		return ccs_print_path_number_acl(head, acl, cond);
 	}
 	if (acl_type == CCS_TYPE_ENV_ACL) {
@@ -1667,14 +1629,12 @@ static bool ccs_print_entry(struct ccs_io_buffer *head,
 	}
 	if (acl_type == CCS_TYPE_CAPABILITY_ACL) {
 		struct ccs_capability_acl *acl
-			= container_of(ptr, struct ccs_capability_acl,
-				       head);
+			= container_of(ptr, struct ccs_capability_acl, head);
 		return ccs_print_capability_acl(head, acl, cond);
 	}
 	if (acl_type == CCS_TYPE_IP_NETWORK_ACL) {
 		struct ccs_ip_network_acl *acl
-			= container_of(ptr, struct ccs_ip_network_acl,
-				       head);
+			= container_of(ptr, struct ccs_ip_network_acl, head);
 		return ccs_print_network_acl(head, acl, cond);
 	}
 	if (acl_type == CCS_TYPE_SIGNAL_ACL) {
@@ -1687,22 +1647,6 @@ static bool ccs_print_entry(struct ccs_io_buffer *head,
 			= container_of(ptr, struct ccs_mount_acl, head);
 		return ccs_print_mount_acl(head, acl, cond);
 	}
-	if (acl_type == CCS_TYPE_UMOUNT_ACL) {
-		struct ccs_umount_acl *acl
-			= container_of(ptr, struct ccs_umount_acl, head);
-		return ccs_print_umount_acl(head, acl, cond);
-	}
-	if (acl_type == CCS_TYPE_CHROOT_ACL) {
-		struct ccs_chroot_acl *acl
-			= container_of(ptr, struct ccs_chroot_acl, head);
-		return ccs_print_chroot_acl(head, acl, cond);
-	}
-	if (acl_type == CCS_TYPE_PIVOT_ROOT_ACL) {
-		struct ccs_pivot_root_acl *acl
-			= container_of(ptr, struct ccs_pivot_root_acl,
-				       head);
-		return ccs_print_pivot_root_acl(head, acl, cond);
-	}
 	BUG(); /* This must not happen. */
 	return false;
 }
@@ -1735,9 +1679,9 @@ static void ccs_read_domain_policy(struct ccs_io_buffer *head)
 			continue;
 		/* Print domainname and flags. */
 		if (domain->quota_warned)
-			quota_exceeded = "quota_exceeded\n";
+			quota_exceeded = CCS_KEYWORD_QUOTA_EXCEEDED "\n";
 		if (domain->domain_transition_failed)
-			transition_failed = "transition_failed\n";
+			transition_failed = CCS_KEYWORD_TRANSITION_FAILED "\n";
 		if (domain->ignore_global_allow_read)
 			ignore_global_allow_read
 				= CCS_KEYWORD_IGNORE_GLOBAL_ALLOW_READ "\n";
@@ -1871,8 +1815,10 @@ static void ccs_read_pid(struct ccs_io_buffer *head)
 	struct ccs_domain_info *domain = NULL;
 	u32 ccs_flags = 0;
 	/* Accessing write_buf is safe because head->io_sem is held. */
-	if (!buf)
+	if (!buf) {
+		head->read_eof = true;
 		return; /* Do nothing if open(O_RDONLY). */
+	}
 	if (head->read_avail || head->read_eof)
 		return;
 	head->read_eof = true;
@@ -1881,12 +1827,12 @@ static void ccs_read_pid(struct ccs_io_buffer *head)
 	if (ccs_str_starts(&buf, "global-pid "))
 		global_pid = true;
 	pid = (unsigned int) simple_strtoul(buf, NULL, 10);
-	read_lock(&tasklist_lock);
+	ccs_tasklist_lock();
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
 	if (global_pid)
-		p = find_task_by_pid_ns(pid, &init_pid_ns);
+		p = ccsecurity_exports.find_task_by_pid_ns(pid, &init_pid_ns);
 	else
-		p = find_task_by_vpid(pid);
+		p = ccsecurity_exports.find_task_by_vpid(pid);
 #else
 	p = find_task_by_pid(pid);
 #endif
@@ -1894,7 +1840,7 @@ static void ccs_read_pid(struct ccs_io_buffer *head)
 		domain = ccs_task_domain(p);
 		ccs_flags = p->ccs_flags;
 	}
-	read_unlock(&tasklist_lock);
+	ccs_tasklist_unlock();
 	if (!domain)
 		return;
 	if (!task_info)
@@ -2114,7 +2060,7 @@ static struct ccs_condition *ccs_get_execute_condition(struct ccs_execve_entry
 			len += strlen(argv0) + 16;
 		}
 	}
-	buf = kmalloc(len, GFP_KERNEL);
+	buf = kmalloc(len, CCS_GFP_FLAGS);
 	if (!buf) {
 		kfree(realpath);
 		return NULL;
@@ -2160,7 +2106,7 @@ static struct ccs_condition *ccs_get_symlink_condition(struct ccs_request_info
 		symlink = r->obj->symlink_target->name;
 		len += strlen(symlink) + 18;
 	}
-	buf = kmalloc(len, GFP_KERNEL);
+	buf = kmalloc(len, CCS_GFP_FLAGS);
 	if (!buf)
 		return NULL;
 	snprintf(buf, len - 1, "if");
@@ -2208,9 +2154,9 @@ static atomic_t ccs_query_observers = ATOMIC_INIT(0);
  * @fmt:     The printf()'s format string, followed by parameters.
  *
  * Returns 0 if the supervisor decided to permit the access request which
- * violated the policy in enforcing mode, 1 if the supervisor decided to
- * retry the access request which violated the policy in enforcing mode,
- * 0 if it is not in enforcing mode, -EPERM otherwise.
+ * violated the policy in enforcing mode, CCS_RETRY_REQUEST if the supervisor
+ * decided to retry the access request which violated the policy in enforcing
+ * mode, 0 if it is not in enforcing mode, -EPERM otherwise.
  */
 int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
 {
@@ -2233,7 +2179,7 @@ int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
 		va_start(args, fmt);
 		len = vsnprintf((char *) &pos, sizeof(pos) - 1, fmt, args) + 4;
 		va_end(args);
-		buffer = kmalloc(len, GFP_KERNEL);
+		buffer = kmalloc(len, CCS_GFP_FLAGS);
 		if (!buffer)
 			return 0;
 		va_start(args, fmt);
@@ -2273,11 +2219,11 @@ int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
 	header = ccs_init_audit_log(&len, r);
 	if (!header)
 		goto out;
-	ccs_query_entry = kzalloc(sizeof(*ccs_query_entry), GFP_KERNEL);
+	ccs_query_entry = kzalloc(sizeof(*ccs_query_entry), CCS_GFP_FLAGS);
 	if (!ccs_query_entry)
 		goto out;
 	len = ccs_round2(len);
-	ccs_query_entry->query = kzalloc(len, GFP_KERNEL);
+	ccs_query_entry->query = kzalloc(len, CCS_GFP_FLAGS);
 	if (!ccs_query_entry->query)
 		goto out;
 	INIT_LIST_HEAD(&ccs_query_entry->list);
@@ -2319,7 +2265,7 @@ int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
 	spin_unlock(&ccs_query_list_lock);
 	switch (ccs_query_entry->answer) {
 	case 3: /* Asked to retry by administrator. */
-		error = 1;
+		error = CCS_RETRY_REQUEST;
 		r->retry++;
 		break;
 	case 1:
@@ -2410,7 +2356,7 @@ static void ccs_read_query(struct ccs_io_buffer *head)
 		head->read_step = 0;
 		return;
 	}
-	buf = kzalloc(len, GFP_KERNEL);
+	buf = kzalloc(len, CCS_GFP_FLAGS);
 	if (!buf)
 		return;
 	pos = 0;
@@ -2486,7 +2432,7 @@ static void ccs_read_version(struct ccs_io_buffer *head)
 {
 	if (head->read_eof)
 		return;
-	ccs_io_printf(head, "1.7.0");
+	ccs_io_printf(head, "1.7.2");
 	head->read_eof = true;
 }
 
@@ -2517,7 +2463,7 @@ static void ccs_read_self_domain(struct ccs_io_buffer *head)
  */
 int ccs_open_control(const u8 type, struct file *file)
 {
-	struct ccs_io_buffer *head = kzalloc(sizeof(*head), GFP_KERNEL);
+	struct ccs_io_buffer *head = kzalloc(sizeof(*head), CCS_GFP_FLAGS);
 	if (!head)
 		return -ENOMEM;
 	mutex_init(&head->io_sem);
@@ -2533,12 +2479,9 @@ int ccs_open_control(const u8 type, struct file *file)
 		break;
 #ifdef CONFIG_CCSECURITY_AUDIT
 	case CCS_GRANTLOG: /* /proc/ccs/grant_log */
-		head->poll = ccs_poll_grant_log;
-		head->read = ccs_read_grant_log;
-		break;
 	case CCS_REJECTLOG: /* /proc/ccs/reject_log */
-		head->poll = ccs_poll_reject_log;
-		head->read = ccs_read_reject_log;
+		head->poll = ccs_poll_audit_log;
+		head->read = ccs_read_audit_log;
 		break;
 #endif
 	case CCS_SELFDOMAIN: /* /proc/ccs/self_domain */
@@ -2593,7 +2536,7 @@ int ccs_open_control(const u8 type, struct file *file)
 		/* Don't allocate read_buf for poll() access. */
 		if (!head->readbuf_size)
 			head->readbuf_size = 4096;
-		head->read_buf = kzalloc(head->readbuf_size, GFP_KERNEL);
+		head->read_buf = kzalloc(head->readbuf_size, CCS_GFP_FLAGS);
 		if (!head->read_buf) {
 			kfree(head);
 			return -ENOMEM;
@@ -2607,7 +2550,7 @@ int ccs_open_control(const u8 type, struct file *file)
 		head->write = NULL;
 	} else if (head->write) {
 		head->writebuf_size = 4096;
-		head->write_buf = kzalloc(head->writebuf_size, GFP_KERNEL);
+		head->write_buf = kzalloc(head->writebuf_size, CCS_GFP_FLAGS);
 		if (!head->write_buf) {
 			kfree(head->read_buf);
 			kfree(head);
@@ -2616,7 +2559,7 @@ int ccs_open_control(const u8 type, struct file *file)
 	}
 	if (type != CCS_QUERY &&
 	    type != CCS_GRANTLOG && type != CCS_REJECTLOG)
-		head->reader_idx = ccs_read_lock();
+		head->reader_idx = ccs_lock();
 	file->private_data = head;
 	/*
 	 * Call the handler now if the file is /proc/ccs/self_domain
@@ -2644,7 +2587,7 @@ int ccs_open_control(const u8 type, struct file *file)
  * Waits for read readiness.
  * /proc/ccs/query is handled by /usr/sbin/ccs-queryd and
  * /proc/ccs/grant_log and /proc/ccs/reject_log are handled by
- * /usr/sbin/ccs-auditd.
+ * /usr/sbin/ccs-auditd .
  */
 int ccs_poll_control(struct file *file, poll_table *wait)
 {
@@ -2669,12 +2612,14 @@ int ccs_read_control(struct file *file, char __user *buffer,
 	int len = 0;
 	struct ccs_io_buffer *head = file->private_data;
 	char *cp;
+	int idx;
 	if (!head->read)
 		return -ENOSYS;
 	if (!access_ok(VERIFY_WRITE, buffer, buffer_len))
 		return -EFAULT;
 	if (mutex_lock_interruptible(&head->io_sem))
 		return -EINTR;
+	idx = ccs_read_lock();
 	while (1) {
 		/* Call the policy handler. */
 		head->read(head);
@@ -2683,7 +2628,7 @@ int ccs_read_control(struct file *file, char __user *buffer,
 		if (len || head->poll || head->read_eof)
 			break;
 		len = head->readbuf_size * 2;
-		cp = kzalloc(len, GFP_KERNEL);
+		cp = kzalloc(len, CCS_GFP_FLAGS);
 		if (!cp) {
 			len = -ENOMEM;
 			goto out;
@@ -2705,6 +2650,7 @@ int ccs_read_control(struct file *file, char __user *buffer,
 	head->read_avail -= len;
 	memmove(cp, cp + len, head->read_avail);
  out:
+	ccs_read_unlock(idx);
 	mutex_unlock(&head->io_sem);
 	return len;
 }
@@ -2725,23 +2671,28 @@ int ccs_write_control(struct file *file, const char __user *buffer,
 	int error = buffer_len;
 	int avail_len = buffer_len;
 	char *cp0 = head->write_buf;
+	int idx;
 	if (!head->write)
 		return -ENOSYS;
 	if (!access_ok(VERIFY_READ, buffer, buffer_len))
 		return -EFAULT;
+	if (mutex_lock_interruptible(&head->io_sem))
+		return -EINTR;
+	idx = ccs_read_lock();
 	/* Don't allow updating policies by non manager programs. */
 	if (head->write != ccs_write_pid &&
 	    head->write != ccs_write_domain_policy &&
-	    !ccs_is_policy_manager())
+	    !ccs_is_policy_manager()) {
+		ccs_read_unlock(idx);
+		mutex_unlock(&head->io_sem);
 		return -EPERM;
-	if (mutex_lock_interruptible(&head->io_sem))
-		return -EINTR;
+	}
 	/* Read a line and dispatch it to the policy handler. */
 	while (avail_len > 0) {
 		char c;
 		if (head->write_avail >= head->writebuf_size - 1) {
 			const int len = head->writebuf_size * 2;
-			char *cp = kzalloc(len, GFP_KERNEL);
+			char *cp = kzalloc(len, CCS_GFP_FLAGS);
 			if (!cp) {
 				error = -ENOMEM;
 				break;
@@ -2766,6 +2717,7 @@ int ccs_write_control(struct file *file, const char __user *buffer,
 		ccs_normalize_line(cp0);
 		head->write(head);
 	}
+	ccs_read_unlock(idx);
 	mutex_unlock(&head->io_sem);
 	return error;
 }
@@ -2789,7 +2741,7 @@ int ccs_close_control(struct file *file)
 		atomic_dec(&ccs_query_observers);
 	if (type != CCS_QUERY &&
 	    type != CCS_GRANTLOG && type != CCS_REJECTLOG)
-		ccs_read_unlock(head->reader_idx);
+		ccs_unlock(head->reader_idx);
 	/* Release memory used for policy I/O. */
 	kfree(head->read_buf);
 	head->read_buf = NULL;
@@ -2802,3 +2754,8 @@ int ccs_close_control(struct file *file)
 		ccs_run_gc();
 	return 0;
 }
+
+void __init ccs_policy_io_init(void)
+{
+	ccsecurity_ops.check_profile = ccs_check_profile;
+}
