@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/proc_if.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1   2009/11/11
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -18,6 +18,73 @@
 #include <linux/version.h>
 #include "internal.h"
 
+/**
+ * ccs_write_transition - write() for /proc/ccs/.transition interface.
+ *
+ * @file:  Pointer to "struct file".
+ * @buf:   Domainname to transit to. Must ends with '\0'.
+ * @count: Size of @buf.
+ * @ppos:  Unused.
+ *
+ * Returns @count on success, negative value otherwise.
+ */
+static ssize_t ccs_write_transition(struct file *file, const char __user *buf,
+				    size_t count, loff_t *ppos)
+{
+	const char *self_domain = ccs_current_domain()->domainname->name;
+	const int self_domain_len = strlen(self_domain);
+	char *data;
+	int data_len;
+	char *tmp;
+	int idx;
+	int error = -ENOMEM;
+	if (!count || count + self_domain_len >= CCS_EXEC_TMPSIZE - 10)
+		return -ENOMEM;
+	data = kmalloc(count, CCS_GFP_FLAGS);
+	if (!data)
+		return -ENOMEM;
+	if (copy_from_user(data, buf, count)) {
+		error = -EFAULT;
+		goto out;
+	}
+	if (memchr(data, '\0', count) != data + count - 1) {
+		error = -EINVAL;
+		goto out;
+	}
+	tmp = ccs_encode(data);
+	kfree(data);
+	data = tmp;
+	if (!data)
+		goto out;
+	data_len = strlen(data);
+	tmp = kzalloc(self_domain_len + data_len + 5, CCS_GFP_FLAGS);
+	if (!tmp)
+		goto out;
+	/*
+	 * Add "//" prefix to requested name in order to distinguish domain
+	 * transitions with execve().
+	 */
+	snprintf(tmp, self_domain_len + data_len + 4, "%s //%s", self_domain,
+		 data);
+	kfree(data);
+	data = tmp;
+	idx = ccs_read_lock();
+	error = ccs_may_transit(data, data + self_domain_len + 1);
+	ccs_read_unlock(idx);
+ out:
+	kfree(data);
+	return error ? error : count;
+}
+
+/* Operations for /proc/ccs/.transition interface. */
+static
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 17)
+const
+#endif
+struct file_operations ccs_transition_operations = {
+	.write = ccs_write_transition,
+};
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 23)
 #if !defined(RHEL_VERSION) || RHEL_VERSION != 3 || !defined(RHEL_UPDATE) || RHEL_UPDATE != 9
 /**
@@ -122,6 +189,9 @@ struct file_operations ccs_operations = {
 };
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+
+struct iattr;
+
 /**
  * proc_notify_change - Update inode's attributes and reflect to the dentry.
  *
@@ -175,7 +245,8 @@ static struct inode_operations ccs_file_inode_operations;
  * Returns nothing.
  */
 static void __init ccs_create_entry(const char *name, const mode_t mode,
-				    struct proc_dir_entry *parent, const u8 key)
+				    struct proc_dir_entry *parent,
+				    const u8 key)
 {
 	struct proc_dir_entry *entry = create_proc_entry(name, mode, parent);
 	if (entry) {
@@ -196,7 +267,7 @@ static void __init ccs_create_entry(const char *name, const mode_t mode,
  *
  * Returns 0.
  */
-static int __init ccs_proc_init(void)
+static void __init ccs_proc_init(void)
 {
 	struct proc_dir_entry *ccs_dir = proc_mkdir("ccs", NULL);
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
@@ -223,11 +294,38 @@ static int __init ccs_proc_init(void)
 	ccs_create_entry("version",          0400, ccs_dir, CCS_VERSION);
 	ccs_create_entry(".execute_handler", 0666, ccs_dir,
 			 CCS_EXECUTE_HANDLER);
+	{
+		struct proc_dir_entry *e = create_proc_entry(".transition",
+							     0222, ccs_dir);
+		if (e)
+			e->proc_fops = &ccs_transition_operations;
+	}
+}
+
+static int __init ccs_init_module(void)
+{
+	if (ccsecurity_ops.disabled)
+		return -EINVAL;
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 0)
+	MOD_INC_USE_COUNT;
+#endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
+	if (init_srcu_struct(&ccs_ss))
+		panic("Out of memory.");
+#endif
+	ccs_proc_init();
+	ccs_mm_init();
+	ccs_capability_init();
+	ccs_file_init();
+	ccs_network_init();
+	ccs_signal_init();
+	ccs_mount_init();
+	ccs_maymount_init();
+	ccs_autobind_init();
+	ccs_policy_io_init();
+	ccs_domain_init();
 	return 0;
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-__initcall(ccs_proc_init);
-#else
-core_initcall(ccs_proc_init);
-#endif
+MODULE_LICENSE("GPL");
+module_init(ccs_init_module);
